/// <reference path="../native.ts" />
/// <reference path="../object.ts" />

namespace trusted.pki {

    const DEFAULT_DATA_FORMAT: DataFormat = DataFormat.PEM;
    const DEFAULT_ENC_ALG: EncryptAlg = EncryptAlg.GOST_28147;
    /**
     * Encrypt and decrypt operations
     *
     * @export
     * @class Cipher
     * @extends {BaseObject<native.PKI.Cipher>}
     */
    export class Cipher extends BaseObject<native.PKI.Cipher> {

        /**
         * Creates an instance of Cipher.
         *
         *
         * @memberOf Cipher
         */
        constructor() {
            super();
            this.handle = new native.PKI.Cipher();
        }

        /**
         * Set provider algorithm(GOST)
         *
         * @param method gost2001, gost2012_256 or gost2012_512
         *
         * @memberOf Cipher
         */
        set ProvAlgorithm(name: string) {
            this.handle.setProvAlgorithm(name);
        }

        /**
         * Encrypt data
         *
         * @param {string} filenameSource This file will encrypted
         * @param {string} filenameEnc File path for save encrypted data
         * @param {EncryptAlg} [alg]
         * @param {DataFormat} [format]
         *
         * @memberOf Cipher
         */
        public encrypt(filenameSource: string, filenameEnc: string,
            alg: EncryptAlg = DEFAULT_ENC_ALG, format: DataFormat = DEFAULT_DATA_FORMAT): void {
                this.handle.encrypt(filenameSource, filenameEnc, alg, format);
        }

        /**
         * Encrypt data asynchronously
         *
         * @param {string} filenameSource This file will encrypted
         * @param {string} filenameEnc File path for save encrypted data
         * @param {(msg: string) => void} done Done callback
         * @param {EncryptAlg} [alg]
         * @param {DataFormat} [format]
         *
         * @memberOf Cipher
         */
        public encryptAsync(filenameSource: string, filenameEnc: string,
            done: (msg: string) => void,
            alg: EncryptAlg = DEFAULT_ENC_ALG, format: DataFormat = DEFAULT_DATA_FORMAT): void {
                this.handle.encryptAsync(filenameSource, filenameEnc, done, alg, format);
        }

        /**
         * Decrypt data
         *
         * @param {string} filenameEnc This file will decrypt
         * @param {string} filenameDec File path for save decrypted data
         * @param {DataFormat} [format]
         *
         * @memberOf Cipher
         */
        public decrypt(filenameEnc: string, filenameDec: string, format?: DataFormat): void {
            this.handle.decrypt(filenameEnc, filenameDec, format);
        }

        /**
         * Decrypt data asynchronously
         *
         * @param {string} filenameEnc This file will decrypt
         * @param {string} filenameDec File path for save decrypted data
         * @param {(msg: string) => void} done Done callback
         * @param {DataFormat} [format]
         *
         * @memberOf Cipher
         */
        public decryptAsync(filenameEnc: string, filenameDec: string, done: (msg: string) => void, format?: DataFormat): void {
            this.handle.decryptAsync(filenameEnc, filenameDec, done, format);
        }

        /**
         * Add recipients certificates
         *
         * @param {CertificateCollection} certs
         *
         * @memberOf Cipher
         */
        set recipientsCerts(certs: CertificateCollection) {
            this.handle.addRecipientsCerts(certs.handle);
        }

        getRecipientInfos(filenameEnc: string, format: trusted.DataFormat): cms.CertInfoCollection {
            const infos = this.handle.getRecipientInfos(filenameEnc, format);

            if (infos) {
                return cms.CertInfoCollection.wrap<native.CMS.CertInfoCollection, cms.CertInfoCollection>(infos);
            } else {
                return undefined;
            }
        }

        /**
         * Set key usage policy verification
         *
         * @param {"kupNone" | "kupDisableKeyAgree" | "kupDisableKeyEnc" | "kupCheckOneOf"} policyToSet
         *
         * @memberOf Cipher
         */
        set kuPolicy(policyToSet: "kupNone" | "kupDisableKeyAgree" | "kupDisableKeyEnc" | "kupCheckOneOf") {
            this.handle.setKuPolicy(policyToSet);
        }
    }
}
