/// <reference path="../native.ts" />
/// <reference path="../object.ts" />

namespace trusted.utils {
    /**
     * cryptographic service provider (CSP) helper
     * Uses on WIN32 or with CPROCSP
     *
     * @export
     * @class Csp
     * @extends {BaseObject<native.UTILS.Csp>}
     */
    export class Csp extends BaseObject<native.UTILS.Csp> {
        /**
         * Check available provaider for GOST 2001
         *
         * @static
         * @returns {boolean}
         * @memberof Csp
         */
        public static isGost2001CSPAvailable(): boolean {
            if(!native) return false;
            const csp = new native.UTILS.Csp();
            return csp.isGost2001CSPAvailable();
        }

        /**
         * Check available provaider for GOST 2012-256
         *
         * @static
         * @returns {boolean}
         * @memberof Csp
         */
        public static isGost2012_256CSPAvailable(): boolean {
            if (!native) return false;
            const csp = new native.UTILS.Csp();
            return csp.isGost2012_256CSPAvailable();
        }

        /**
         * Check available provaider for GOST 2012-512
         *
         * @static
         * @returns {boolean}
         * @memberof Csp
         */
        public static isGost2012_512CSPAvailable(): boolean {
            if (!native) return false;
            const csp = new native.UTILS.Csp();
            return csp.isGost2012_512CSPAvailable();
        }

        /**
         * Verify license for CryptoPro CSP
         * Throw exception if provaider not available
         *
         * @static
         * @returns {boolean}
         * @memberof Csp
         */
        public static checkCPCSPLicense(): boolean {
            if (!native) throw new Error("Unable to check CSP license");
            const csp = new native.UTILS.Csp();
            return csp.checkCPCSPLicense();
        }

        /**
         * Return instaled correct license for CryptoPro CSP
         * Throw exception if provaider not available
         *
         * @static
         * @returns {boolean}
         * @memberof Csp
         */
        public static getCPCSPLicense(): string {
            if (!native) throw new Error("Unable to get CSP license");
            const csp = new native.UTILS.Csp();
            return csp.getCPCSPLicense();
        }

        /**
         * Return instaled correct version for CryptoPro CSP
         * Throw exception if provaider not available
         *
         * @static
         * @returns {boolean}
         * @memberof Csp
         */
        public static getCPCSPVersion(): string {
            if (!native) throw new Error("Unable to get CSP version");
            const csp = new native.UTILS.Csp();
            return csp.getCPCSPVersion();
        }

        public static getCPCSPVersionPKZI(): string {
            if (!native) throw new Error("Unable to get PKZI version");
            const csp = new native.UTILS.Csp();
            return csp.getCPCSPVersionPKZI();
        }

        public static getCPCSPVersionSKZI(): string {
            if (!native) throw new Error("Unable to get SKZI version");
            const csp = new native.UTILS.Csp();
            return csp.getCPCSPVersionSKZI();
        }

        public static getCPCSPSecurityLvl(): string {
            if (!native) throw new Error("Unable to get CSP security level");
            const csp = new native.UTILS.Csp();
            return csp.getCPCSPSecurityLvl();
        }

        /**
        * Enumerate available CSP
        *
        * @static
        * @returns {object[]} {type: nuber, name: string}
        * @memberof Csp
        */
        public static enumProviders(): object[] {
            if (!native) return [];
            const csp = new native.UTILS.Csp();
            return csp.enumProviders();
        }

        /**
         * Enumerate conainers
         *
         * @static
         * @param {number} [type]
         * @returns {string[]} Fully Qualified Container Name
         * @memberof Csp
         */
        public static enumContainers(type: null, provName = ""): native.UTILS.IContainerName[] {
            const csp = new native.UTILS.Csp();
            return csp.enumContainers(type, provName);
        }

        /**
         * Get certificate by container and provider props
         *
         * @static
         * @param {string} contName
         * @param {number} provType
         * @param {string} [provName=""]
         * @returns {pki.Certificate}
         * @memberof Csp
         */
        // tslint:disable-next-line:max-line-length
        public static getCertificateFromContainer(contName: string, provType: number, provName: string = ""): pki.Certificate {
            const cert: pki.Certificate = new pki.Certificate();
            const csp = new native.UTILS.Csp();
            cert.handle = csp.getCertificateFromContainer(contName, provType, provName);
            return cert;
        }

        public static installCertificateFromContainer(contName: string, provType: number, provName = ""): void {
            const csp = new native.UTILS.Csp();
            csp.installCertificateFromContainer(contName, provType, provName);
            return;
        }

        public static installCertificateToContainer(cert: pki.Certificate, contName: string,
            provType: number, provName = ""): void {
            const csp = new native.UTILS.Csp();
            csp.installCertificateToContainer(cert.handle, contName, provType, provName);
            return;
        }

        public static deleteContainer(contName: string, provType: number, provName = ""): void {
            const csp = new native.UTILS.Csp();
            csp.deleteContainer(contName, provType, provName);
            return;
        }

        /**
         * Get container name by certificate
         *
         * @static
         * @param {pki.Certificate} cert
         * @param {string} [category="MY"]
         * @returns {string}
         * @memberof Csp
         */
        public static getContainerNameByCertificate(cert: pki.Certificate, category: string = "MY"): string {
            const csp = new native.UTILS.Csp();
            return csp.getContainerNameByCertificate(cert.handle, category);
        }

        /**
         * Ensure that the certificate's private key is available
         *
         * @static
         * @param {Certificate} cert
         * @returns {boolean}
         * @memberOf Csp
         */
        public hasPrivateKey(cert: pki.Certificate): boolean {
            return this.handle.hasPrivateKey(cert.handle);
        }

        public static buildChain(cert: pki.Certificate): pki.CertificateCollection {
            const csp = new native.UTILS.Csp();

            const certscol: pki.CertificateCollection =
                new pki.CertificateCollection(csp.buildChain(cert.handle));

            return certscol;
        }

        public static buildChainAsync(cert: pki.Certificate, done: (error: string, certs: pki.CertificateCollection) => void): void {
            const csp = new native.UTILS.Csp();

            csp.buildChainAsync(cert.handle, function (error: string, certs: native.PKI.CertificateCollection) {
                if (error) {
                    done(error, null);
                    return;
                }

                const certscol: pki.CertificateCollection = new pki.CertificateCollection(certs);
                done(null, certscol);
            });
        }

        public static verifyCertificateChain(cert: pki.Certificate): boolean {
            const csp = new native.UTILS.Csp();
            return csp.verifyCertificateChain(cert.handle);
        }

        public static verifyCertificateChainAsync(cert: pki.Certificate, done: (error: string, result: boolean) => void): void {
            const csp = new native.UTILS.Csp();
            return csp.verifyCertificateChainAsync(cert.handle, done);
        }

        public static verifyCRL(crl: pki.CRL): boolean {
            const csp = new native.UTILS.Csp();
            return csp.verifyCRL(crl.handle);
        }

        /**
         * Find certificate in MY store and check that private key exportable
         *
         * @static
         * @param {pki.Certificate} cert
         * @returns {boolean}
         * @memberof Csp
         */
        public static isHaveExportablePrivateKey(cert: pki.Certificate): boolean {
            const csp = new native.UTILS.Csp();
            return csp.isHaveExportablePrivateKey(cert.handle);
        }

        /**
         * Create Pkcs by cert
         * NOTE:  only for certificates with exportable key. Check it by isHaveExportablePrivateKey
         *
         * @static
         * @param {pki.Certificate} cert
         * @param {boolean} exportPrivateKey
         * @param {string} [password]
         * @returns {pki.PKCS12}
         * @memberof Csp
         */
        public static certToPkcs12(cert: pki.Certificate, exportPrivateKey: boolean, password?: string): pki.PKCS12 {
            const csp = new native.UTILS.Csp();

            return pki.PKCS12.wrap<native.PKI.PKCS12, pki.PKCS12>(csp.certToPkcs12(cert.handle, exportPrivateKey, password));
        }

        /**
         * Import PFX to store
         *
         * @static
         * @param {pki.PKCS12} p12
         * @param {string} [password]
         * @returns {void}
         * @memberof Csp
         */
        public static importPkcs12(p12: pki.PKCS12, password?: string): void {
            const csp = new native.UTILS.Csp();

            csp.importPkcs12(p12.handle, password);
            return;
        }

        /**
         * Creates an instance of Csp.
         *
         *
         * @memberOf Csp
         */
        constructor() {
            super();
            this.handle = new native.UTILS.Csp();
        }
    }
}
