/// <reference path="../native.ts" />
/// <reference path="../object.ts" />

namespace trusted.utils {
    /**
     * Object for calculating data hash
     *
     * @export
     * @class Hash
     * @extends {BaseObject<native.UTILS.Hash>}
     */
    export class Hash extends BaseObject<native.UTILS.Hash> {
        /**
         * Add data to hash
         *
         * @param {Buffer} buffer Buffer with data to add into hash
         * @memberof Hash
         */
        public addData(buffer: Buffer): void {
            this.handle.addData(buffer);
        }

        /**
         * Get value of hashed data
         *
         * @returns {Buffer} Buffer with hash value
         * @memberof Hash
         */
        public getValue(): Buffer {
            return this.handle.getValue();
        }

        /**
         * Hash data from buffer
         *
         * @static
         * @param {trusted.HashAlg} hash_alg Hash algorithm ID
         * @param {Buffer} data Buffer with data to hash
         * @returns {Buffer}
         * @memberof Hash
         */
        public static hashData(hash_alg: trusted.HashAlg, data: Buffer | string): Buffer {
            const hash = new native.UTILS.Hash();
            return hash.hashData(hash_alg, data);
        }

        /**
         * Hash data from buffer asynchronously
         *
         * @static
         * @param {trusted.HashAlg} hash_alg Hash algorithm ID
         * @param {Buffer} data Buffer with data to hash
         * @returns {Buffer}
         * @memberof Hash
         */
        public static hashDataAsync(hash_alg: trusted.HashAlg, data: Buffer | string, done: (error: string, hashValue: Buffer) => void): Buffer {
            const hash = new native.UTILS.Hash();
            return hash.hashDataAsync(hash_alg, data, done);
        }

        /**
         * Creates an instance of Hash.
         *
         * @param {trusted.HashAlg} hash_alg Hash algorithm ID
         *
         * @memberOf Hash
         */
        constructor(hash_alg: trusted.HashAlg) {
            super();
            if (undefined === hash_alg || null === hash_alg) {
                throw new TypeError("Hash::constructor: Wrong input param");
            }

            this.handle = new native.UTILS.Hash(hash_alg);
        }
    }
}
